

#if !defined __CAMERA_AI_H
#define __CAMERA_AI_H


#include "vissim.h"
#include "timer.h"
#include "user_io.h"


#define PS_CAMERA_MODE_FOLLOW  0x00
#define PS_CAMERA_MODE_MIMIC   0x01
#define PS_CAMERA_MODE_ORBIT   0x02
#define PS_CAMERA_MODE_FREEFLY 0x03
#define PS_CAMERA_MODE_COCKPIT 0x04


#define KEYSTROKE_STOP            19
#define KEYSTROKE_VIEW_FORWARD    20
#define KEYSTROKE_VIEW_BACKWARD   21
#define KEYSTROKE_VIEW_CHASE      22
#define KEYSTROKE_INERTIA_TOGGLE  23
#define KEYSTROKE_THROTTLE_TOGGLE 24
#define KEYSTROKE_SUBTARGET_NEXT  25
#define KEYSTROKE_TARGET_NEXT     26
#define KEYSTROKE_TARGET_NEXT_ENEMY     27
#define KEYSTROKE_TARGET_NEAREST_ENEMY  28
#define KEYSTROKE_TARGET_CENTER         29
#define KEYSTROKE_SELECT_NEXT_PRIMARY   30
#define KEYSTROKE_SELECT_NEXT_SECONDARY 31
#define KEYSTROKE_FULL_PRIMARY          32
#define KEYSTROKE_ROUND_ROBIN_PRIMARY   33
#define KEYSTROKE_AFTERBURNER           34
#define KEYSTROKE_MATCH_SPEED           35
#define KEYSTROKE_CYCLE_STATUS          36


/* **************************************************************
************************************************************** */
class camera_ai {

   protected: 
      unsigned int mode;
      vector4f xmx[4];
      double orbit_rad;
      
      void set_stat();
      void face_target();

      void freefly(iface *iman, user_io *userinput, timer_type *timer);
      void followfly(timer_type *timer);
      void orbitfly(timer_type *timer);
      void mimicfly(timer_type *timer);
      void cockpitfly(timer_type *timer);
      
      virtual int nix_tunneling(double *x, double *y, double *z) = 0;
      virtual int clamp_tunneling(double *x, double *y, double *z) = 0;
      virtual void process_user_io(user_io *userinput, timer_type *timer);
      
   public:
      basic_platform *attach_object;
      vector3d rpy, xyz;
      double speed;

      void init();

      camera_ai() { init(); }

      void set_mode(unsigned int m);
      int get_mode() { return mode; }

      void get_xmx(vector4f *mx) { copymx4x4o(mx, xmx); }
      void set_xmx(vector4f *mx) { copymx4x4o(xmx, mx); }

      virtual void animate_camera(iface *iman, user_io *userinput, timer_type *timer);
};

#endif
