

#if !defined __VGEV_SYMBOL_H
#define __VGEV_SYMBOL_H


#include "vgev_draw.h"
#include "vgev_icon.h"

#define DRAWABLEFLAG_NULL       0x00000000
#define DRAWABLEFLAG_VISIBLE    0x00000001
#define DRAWABLEFLAG_HIGHLIGHT  0x00000002
#define DRAWABLEFLAG_PICK       0x00000004

#define DRAWABLEFLAG_PLATFORM   0x00010000
#define DRAWABLEFLAG_CAMERA     0x00020000
#define DRAWABLEFLAG_OVERLAY    0x00040000

#define SYMBOLFLAG_NULL         0x00000000
#define SYMBOLFLAG_RECALC       0x00000001

#define OVERLAY_POLYGON         0x01


/* **************************************************************
************************************************************** */
class drawable_symbol : public dbl_llist {

   public:
      int id;
      vector4uc color;
      icon_type *icon;

      double lat, lon, alt;
      int agl_flag;

      int x, y;
      unsigned int flags;
      void *source;

      drawable_symbol();
      virtual ~drawable_symbol() {}
      
      virtual void render(canvas *tgev);
      virtual void recalc(canvas *drawable);
      virtual void set_pick(int pickflag, canvas *drawable);
};


/* **************************************************************
************************************************************** */
class platform_symbol : public drawable_symbol {

   public:
      virtual ~platform_symbol() {}

      void render(canvas *tgev);
};


/* **************************************************************
************************************************************** */
class drawable_manager : public dbl_llist_manager {

   public:
   
      unsigned int flags;
      
      drawable_manager() { flags = SYMBOLFLAG_RECALC; }
      virtual ~drawable_manager() {}

      void recalc(canvas *drawable);
      void set_pick(drawable_symbol *symbol, canvas *drawable);
      drawable_symbol *find_object(int bx, int by, drawable_symbol **pick, unsigned int mask);
      void resort();
};


/* **************************************************************
************************************************************** */
class overlay_symbol : public drawable_symbol {

   public:
      void render(canvas *tgev);
};


/* **************************************************************
************************************************************** */
class overlay_point : public dbl_llist {

   public:
      overlay_symbol *symbol;

      overlay_point();
      virtual ~overlay_point();
};


/* **************************************************************
************************************************************** */
class overlay_node : public dbl_llist {

   public:
      int id;
      vector4uc color;

      overlay_node() { id = 0; color[0] = color[1] = color[2] = 0; }
      virtual ~overlay_node() {}

      virtual int parse(FILE *infile, char *token) = 0;
      virtual int dump(FILE *outfile) = 0;
      virtual overlay_node *clone() = 0;
      virtual int query_whatami() = 0;
};


/* **************************************************************
************************************************************** */
class overlay_polygon_node : public overlay_node {

   public:
      dbl_llist_manager point_list;
      int fillflag;

      overlay_polygon_node() { fillflag = 0; }
      overlay_polygon_node(int x) { fillflag = x; }
      virtual ~overlay_polygon_node() {}

      int parse(FILE *infile, char *token);
      int dump(FILE *outfile);
      overlay_node *clone();
      int query_whatami() { return OVERLAY_POLYGON; }
};


/* **************************************************************
************************************************************** */
class overlay_point_node : public dbl_llist {

   public:
      vector2f latlon;

      virtual ~overlay_point_node() {}
};


/* **************************************************************
************************************************************** */
class overlay_primitive : public dbl_llist {

   public:
      vector4uc color;
      int id;
      string_type filename;

      virtual ~overlay_primitive() {}

      virtual int parse(FILE *infile, char *token);
      virtual int parse(overlay_node *node);
      virtual void render(canvas *tgev) = 0;
};


/* **************************************************************
************************************************************** */
class overlay_polyline : public overlay_primitive {

   protected:
      int fillflag;

   public:
      dbl_llist_manager list;

      overlay_polyline() { fillflag = 0; }
      overlay_polyline(int x) { fillflag = x; }
      virtual ~overlay_polyline() {}

      int parse(FILE *infile, char *token);
      int parse(overlay_node *node);
      void render(canvas *tgev);
};


#endif

