

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File:           name2sem.c                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:    This file contains the functions, methods, and global
      variables needed to extract and correlate ELEMENT names w/ semantic
      codes in the TDB
			
   Written by:	Alex Harper
*********************************************************************** */


#include "dataio.h"
#include "global.h"
#include "pstring.h"

#include "parse_tdb.h"


/* ************************************************************
  This function correlates element semantic codes w/ element names

  char *filename  // filename of the semantic code correlation file
************************************************************ */
int read_name2sem(char *filename, dbl_llist_manager *semcode_table) {

   sfile sinfile;
   char token[MAXSTRLEN];
   name2sem *ptr;
      
   if (!sinfile.scan_data(filename, NULL, PLATFORM_SLASH))
      return 0;

   while (sinfile.scan_token(token) && strcmp(token, "ALLOCATION-OPTIONS"));

   sinfile.scan_token(token);
   sinfile.scan_token(token);
   
   if (sinfile.seof())
      return 1;

   while (sinfile.scan_token(token)) {
      
      if (!strcmp(token, "PLATFORMS") || !strcmp(token, TOKEN_SNR_RCVR) || !strcmp(token, TOKEN_SNR_XMTR)) {

         sinfile.scan_token(token);
         sinfile.scan_token(token);

         while (sinfile.scan_token(token) && 
             strcmp(token, "PLAYERS") &&
             strcmp(token, "MESSAGE-NAME") &&
             strcmp(token, TOKEN_MOVER) &&
             strcmp(token, TOKEN_DISRUPTOR) &&
             strcmp(token, "SIGNATURE-NAME") &&
             strcmp(token, "GROUPS") &&
             strcmp(token, "MESSAGE-ACTION") &&
             strcmp(token, TOKEN_THINKER) &&
             strcmp(token, "CAPABILITIES") &&
             strcmp(token, TOKEN_SNR_XMTR) &&
             strcmp(token, "COLORS") &&
             strcmp(token, "PARAGRAPH-NUMBER") &&
             strcmp(token, "ELINT-NOTATION") &&
             strcmp(token, "ORDNANCE") &&
             strcmp(token, "NETS") &&
             strcmp(token, "ZONES") &&
             strcmp(token, "COMMAND-CHAINS") &&
             strcmp(token, "SYMBOLS") &&
             strcmp(token, "GEOGRAPHIC-CONTROL") &&
             strcmp(token, "SHAPES") &&
             strcmp(token, "PARAGRAPH-NAME") &&
             strcmp(token, TOKEN_SNR_RCVR) &&
             strcmp(token, "ELEMENTS") &&
             strcmp(token, "GEOGRAPHIC-POINT") &&
             strcmp(token, "FUTURE-PLAYER") &&
             strcmp(token, TOKEN_COMM_XMTR) &&
             strcmp(token, "FACETS") &&
             strcmp(token, "MANEUVERS") &&
             strcmp(token, "SUSCEPTIBILITIES") &&
             strcmp(token, "STANDARD") &&
             strcmp(token, "TACTIC") &&
             strcmp(token, "PLATFORMS") &&
             strcmp(token, "CM-EXPENDABLE") &&
             strcmp(token, TOKEN_WEAPON) &&
             strcmp(token, TOKEN_COMM_RCVR) &&
             strcmp(token, "Memory")) {

            semcode_table->append(ptr = new name2sem, NULL);
            ptr->name.stringcpy(token);
            sinfile.scan_token(token);
            ptr->semcode = atoi(token);
            sinfile.scan_token(token);
         }

      }

      if (sinfile.seof())
         return 1;
   }
   
   return 1;
}
